ITEM_TYPES = {
    'String', 'Number', 'Switch', 'Contact', 'Dimmer', 'Rollershutter',
    'Color', 'DateTime', 'Location', 'Player', 'Group', 'Image',
}

ITEM_DIMENSIONS = {'Length', 'Temperature', 'Pressure', 'Speed', 'Intensity', 'Angle', 'Dimensionless'}

GROUP_FUNCTIONS = {'AND', 'OR', 'NAND', 'NOR', 'AVG', 'MAX', 'MIN', 'SUM'}
